@@ -33,7 +33,7 @@
 #include "persist-state.h"
 #include "template/templates.h"
 #include "host-resolve.h"
-#include "type-hinting.h"
+#include "logmsg/type-hinting.h"
 #include "stats/stats.h"
 #include "dnscache.h"
 #include "file-perms.h"
@@ -62,9 +62,6 @@ struct _GlobalConfig
   /* hex-encoded syslog-ng major/minor, e.g. 0x0201 is syslog-ng 2.1 format */
   gint user_version;
 
-  /* version number as parsed from the configuration file, it can be set
-   * multiple times if the user uses @version multiple times */
-  guint parsed_version;
   const gchar *filename;
   PluginContext plugin_context;
   gboolean use_plugin_discovery;
@@ -94,6 +91,7 @@ struct _GlobalConfig
   gint log_fifo_size;
   gint log_msg_size;
   gboolean trim_large_messages;
+  gint log_level;
 
   gboolean create_dirs;
   FilePermOptions file_perm_options;
@@ -126,22 +124,27 @@ struct _GlobalConfig
   GList *file_list;
 };
 
+gboolean cfg_load_module_with_args(GlobalConfig *cfg, const gchar *module_name, CfgArgs *args);
 gboolean cfg_load_module(GlobalConfig *cfg, const gchar *module_name);
+gboolean cfg_is_module_available(GlobalConfig *self, const gchar *module_name);
+void cfg_discover_candidate_modules(GlobalConfig *self);
 
 Plugin *cfg_find_plugin(GlobalConfig *cfg, gint plugin_type, const gchar *plugin_name);
-gpointer cfg_parse_plugin(GlobalConfig *cfg, Plugin *plugin, YYLTYPE *yylloc, gpointer arg);
+gpointer cfg_parse_plugin(GlobalConfig *cfg, Plugin *plugin, CFG_LTYPE *yylloc, gpointer arg);
 
 gboolean cfg_allow_config_dups(GlobalConfig *self);
 
 void cfg_bad_hostname_set(GlobalConfig *self, gchar *bad_hostname_re);
 gint cfg_lookup_mark_mode(const gchar *mark_mode);
 void cfg_set_mark_mode(GlobalConfig *self, const gchar *mark_mode);
+gboolean cfg_set_log_level(GlobalConfig *self, const gchar *log_level);
 
 gint cfg_tz_convert_value(gchar *convert);
 gint cfg_ts_format_value(gchar *format);
 
+void cfg_set_version_without_validation(GlobalConfig *self, gint version);
 gboolean cfg_set_version(GlobalConfig *self, gint version);
-void cfg_load_candidate_modules(GlobalConfig *self);
+gboolean cfg_set_current_version(GlobalConfig *self);
 
 void cfg_set_global_paths(GlobalConfig *self);
 
@@ -168,11 +171,12 @@ gpointer cfg_persist_config_fetch(GlobalConfig *cfg, const gchar *name);
 
 typedef gboolean(* mangle_callback)(GlobalConfig *cfg, LogMessage *msg, gpointer user_data);
 
-void register_source_mangle_callback(GlobalConfig *src,mangle_callback cb);
-void uregister_source_mangle_callback(GlobalConfig *src,mangle_callback cb);
+void register_source_mangle_callback(GlobalConfig *src, mangle_callback cb);
+gboolean is_source_mangle_callback_registered(GlobalConfig *src, mangle_callback cb);
+void uregister_source_mangle_callback(GlobalConfig *src, mangle_callback cb);
 
 static inline gboolean
-cfg_is_config_version_older(GlobalConfig *cfg, gint req)
+__cfg_is_config_version_older(GlobalConfig *cfg, gint req)
 {
   if (!cfg)
     return FALSE;
@@ -181,6 +185,46 @@ cfg_is_config_version_older(GlobalConfig *cfg, gint req)
   return TRUE;
 }
 
+/* VERSION_VALUE_LAST_SEMANTIC_CHANGE needs to be bumped in versioning.h whenever
+ * we add a conditional on the config version anywhere in the codebase.  The
+ * G_STATIC_ASSERT checks that we indeed did that.
+ *
+ * We also allow merging the features for the upcoming major version.
+ */
+
+#define cfg_is_config_version_older(__cfg, __req) \
+  ({ \
+    /* check that VERSION_VALUE_LAST_SEMANTIC_CHANGE is set correctly */ G_STATIC_ASSERT((__req) <= VERSION_VALUE_LAST_SEMANTIC_CHANGE || (__req) == VERSION_VALUE_NEXT_MAJOR); \
+    __cfg_is_config_version_older(__cfg, __req); \
+  })
+
+/* This function returns TRUE if a version based feature flip is enabled.
+ * This will enable or disable feature related upgrade warnings.
+ *
+ * This was initially introduced for "typing" support, where the following
+ * is implemented:
+ *
+ * As long as we are using a 3.x version number (less than 3.255), the upgrade
+ * warnings are suppressed and compatibility mode is enabled.  Once @version
+ * is something equal or larger than 3.255, we still enable compatibility but
+ * warnings will NOT be suppressed.  This is controlled with
+ * FEATURE_TYPING_MIN_VERSION.
+ *
+ * Once we release 4.0, FEATURE_TYPING_MIN_VERSION needs to be set to zero in
+ * versioning.h With that all upgrade notices start to appear when syslog-ng
+ * finds a 3.x configuration.
+ */
+#define __cfg_is_feature_enabled(cfg, min_version) \
+  ({ \
+    /* the flip-over for min_version reached, set min_version to 0 */ G_STATIC_ASSERT(min_version == 0 || VERSION_VALUE_CURRENT < ((min_version) - 1));       \
+    version_convert_from_user(cfg->user_version) >= (min_version) - 1;  \
+  })
+
+#define cfg_is_feature_enabled(cfg, topic)        __cfg_is_feature_enabled(cfg, FEATURE_ ## topic ## _MIN_VERSION)
+#define cfg_is_typing_feature_enabled(cfg)        cfg_is_feature_enabled(cfg, TYPING)
+
+#define cfg_is_experimental_feature_enabled(cfg)  0
+
 static inline void
 cfg_set_use_uniqid(gboolean flag)
 {
@@ -191,4 +235,17 @@ gint cfg_get_user_version(const GlobalConfig *cfg);
 guint cfg_get_parsed_version(const GlobalConfig *cfg);
 const gchar *cfg_get_filename(const GlobalConfig *cfg);
 
+static inline EVTTAG *
+cfg_format_version_tag(const gchar *tag_name, gint version)
+{
+  return evt_tag_printf(tag_name, "%d.%d", (version & 0xFF00) >> 8, version & 0xFF);
+}
+
+static inline EVTTAG *
+cfg_format_config_version_tag(GlobalConfig *self)
+{
+  return cfg_format_version_tag("config-version", self->user_version);
+}
+
+
 #endif
