@@ -43,9 +43,16 @@ EVTREC *msg_event_create_from_desc(gint prio, const char *desc);
 void msg_event_free(EVTREC *e);
 void msg_event_send(EVTREC *e);
 void msg_event_suppress_recursions_and_send(EVTREC *e);
+void msg_event_print_event_to_stderr(EVTREC *e);
 
 
 void msg_set_post_func(MsgPostFunc func);
+gint msg_map_string_to_log_level(const gchar *log_level);
+void msg_set_log_level(gint new_log_level);
+gint msg_get_log_level(void);
+void msg_apply_cmdline_log_level(gint new_log_level);
+void msg_apply_config_log_level(gint new_log_level);
+
 void msg_init(gboolean interactive);
 void msg_deinit(void);
 
@@ -99,16 +106,19 @@ void msg_add_option_group(GOptionContext *ctx);
             msg_event_create(EVT_PRI_DEBUG, desc, ##tags, NULL ));          \
   } while (0)
 
-#if SYSLOG_NG_ENABLE_DEBUG
 #define msg_trace(desc, tags...)              \
   do {                    \
     if (G_UNLIKELY(trace_flag))                     \
-            msg_event_suppress_recursions_and_send(                               \
-                  msg_event_create(EVT_PRI_DEBUG, desc, ##tags, NULL ));          \
+      msg_event_suppress_recursions_and_send(                               \
+            msg_event_create(EVT_PRI_DEBUG, desc, ##tags, NULL ));          \
+  } while (0)
+
+#define msg_diagnostics(desc, tags...)              \
+  do {                    \
+    if (G_UNLIKELY(trace_flag))                     \
+      msg_event_print_event_to_stderr(              \
+            msg_event_create(EVT_PRI_DEBUG, desc, ##tags, NULL ));          \
   } while (0)
-#else
-#define msg_trace(desc, tags...)
-#endif
 
 #define __once()          \
         ({            \
