@@ -26,7 +26,7 @@
 #define MSG_FORMAT_H_INCLUDED
 
 #include "syslog-ng.h"
-#include "timeutils.h"
+#include "timeutils/zoneinfo.h"
 #include "logproto/logproto-server.h"
 
 #include <regex.h>
@@ -56,6 +56,9 @@ enum
   /* for the date part of a message, only skip it, don't fully parse - recommended for keep_timestamp(no) */
   LP_NO_PARSE_DATE = 0x0400,
   LP_STORE_RAW_MESSAGE = 0x0800,
+  LP_GUESS_TIMEZONE = 0x1000,
+  LP_NO_HEADER = 0x2000,
+  LP_NO_RFC3164_FALLBACK = 0x4000,
 };
 
 typedef struct _MsgFormatHandler MsgFormatHandler;
@@ -70,6 +73,8 @@ typedef struct _MsgFormatOptions
   gchar *recv_time_zone;
   TimeZoneInfo *recv_time_zone_info;
   regex_t *bad_hostname;
+  gchar *sdata_prefix;
+  gsize sdata_prefix_len;
   gint sdata_param_value_max;
 } MsgFormatOptions;
 
@@ -81,9 +86,22 @@ struct _MsgFormatHandler
    */
   LogProtoServer *(*construct_proto)(const MsgFormatOptions *options, LogTransport *transport,
                                      const LogProtoServerOptions *proto_options);
-  void (*parse)(const MsgFormatOptions *options, const guchar *data, gsize length, LogMessage *msg);
+  gboolean (*parse)(const MsgFormatOptions *options, LogMessage *msg,
+                    const guchar *data, gsize length,
+                    gsize *problem_position);
 };
 
+gboolean msg_format_try_parse_into(MsgFormatOptions *options, LogMessage *msg,
+                                   const guchar *data, gsize length,
+                                   gsize *problem_position);
+void msg_format_parse_into(MsgFormatOptions *options, LogMessage *msg,
+                           const guchar *data, gsize length);
+
+LogMessage *msg_format_construct_message(MsgFormatOptions *options, const guchar *data, gsize length);
+LogMessage *msg_format_parse(MsgFormatOptions *options, const guchar *data, gsize length);
+
+void msg_format_options_set_sdata_prefix(MsgFormatOptions *options, const gchar *prefix);
+
 void msg_format_options_defaults(MsgFormatOptions *options);
 void msg_format_options_init(MsgFormatOptions *parse_options, GlobalConfig *cfg);
 void msg_format_options_destroy(MsgFormatOptions *parse_options);
@@ -91,6 +109,4 @@ void msg_format_options_copy(MsgFormatOptions *options, const MsgFormatOptions *
 
 gboolean msg_format_options_process_flag(MsgFormatOptions *options, const gchar *flag);
 
-void msg_format_inject_parse_error(LogMessage *msg, const guchar *data, gsize length, gint problem_position);
-
 #endif
