@@ -60,6 +60,17 @@ struct _LogTemplate
   GList *compiled_template;
   GlobalConfig *cfg;
   guint escape:1, def_inline:1, trivial:1, literal:1;
+
+
+  /* This value stores the type-hint the user _explicitly_ specified.  If
+   * this is an automatic cast to string (in compat mode), this would be
+   * LM_VT_NONE while "type_hint" would be LM_VT_STRING */
+  LogMessageValueType explicit_type_hint;
+
+  /* This is the type-cast we do perform as we evaluate this template.  It
+   * might differ from explicit_type_hint in case we are in compatibility
+   * mode where the template would be cast to a string when the type-hint is
+   * unspecified.  */
   LogMessageValueType type_hint;
 };
 
@@ -68,6 +79,7 @@ struct _LogTemplate
 void log_template_set_escape(LogTemplate *self, gboolean enable);
 gboolean log_template_set_type_hint(LogTemplate *self, const gchar *hint, GError **error);
 gboolean log_template_compile(LogTemplate *self, const gchar *template, GError **error);
+gboolean log_template_compile_with_type_hint(LogTemplate *self, const gchar *template_and_typehint, GError **error);
 void log_template_compile_literal_string(LogTemplate *self, const gchar *literal);
 gboolean log_template_is_literal_string(const LogTemplate *self);
 const gchar *log_template_get_literal_value(const LogTemplate *self, gssize *value_len);
@@ -86,6 +98,7 @@ void log_template_options_clone(LogTemplateOptions *source, LogTemplateOptions *
 void log_template_options_init(LogTemplateOptions *options, GlobalConfig *cfg);
 void log_template_options_destroy(LogTemplateOptions *options);
 void log_template_options_defaults(LogTemplateOptions *options);
+void log_template_options_global_defaults(LogTemplateOptions *options);
 
 void log_template_global_init(void);
 void log_template_global_deinit(void);
